/* OMMailMessage.h created by luke on Sat 18-Jul-1998 */

#import <Foundation/Foundation.h>
#import "FrameworkDefines.h"

//
//  OMMailMessage is a class for sending RFC822 formatted email messages via
//  SMTP (RFC821).  It can handle sender and receiver addresses of the form:
//
//      username@some.domain                 or
//      <username@some.domain>               or
//      "Full Name" <username@some.domain>
//
//  It has built-in support for handling To:, From:, Subject: Cc:, & Bcc: headers.
//  Abitrary header support would be pretty trivial to add, but those are the only
//  headers I needed to support at the time.
//
//  OmniMail.framework depends on OmniNetworking.framework to talk to the SMTP server.
//
//  Miscellaneous:
//      *  We might use ONServiceEntry in the future to locate the smtp server.
//      *  Some day someone might want support for arbitrary headers.
//      *  Might want an option to enable continuing if one of the receiver addresses
//           is invalid.
//
//  IMPORTANT NOTE: OmniMail.framework links against OmniBase.framework, so you should
//  call [OBPostLoader processClasses] in main(), or shortly thereafter.  The OBPostLoader
//  calls all the +didLoads and +performPosings, which gives the various objects in other
//  frameworks (OmniNetworking.framework, for one) a chance to initialize themselves
//  properly.  Using +didLoad is reliable on all supported platforms, whereas using +load
//  is not.
//
//  SOLARIS NOTE: If you use this framework on Solaris, you must be sure that your executable
//  project (be it tool, DO Server, WebObjects app, whatever) links libresolv.so.  If you're
//  working in a multi-platform environment, you can put the following in your executable
//  project Makefile.preamble:
//
//     ifeq ($(PLATFORM_OS), solaris)
//        OTHER_LIBS += -lresolv
//     endif
//
//  This must be in the Makefile.preamble of the *executable* project.  Putting it in a
//  framework Makefile.preamble won't do the job.  Gotta love Solaris.
//

@interface OMMailMessage : NSObject
{
    NSMutableArray *_receiverAddresses;
    NSString *_senderAddress;
    NSMutableArray *_ccAddresses;
    NSMutableArray *_bccAddresses;
    NSString *_subject;
    NSString *_body;
    NSString *_smtpHostName;
    unsigned int _smtpPort;
        // If port isn't specified, it defaults to standard SMTP port: 25
}

+ (void)initialize;
- (id)init;
- (void)dealloc;

- (id)initWithReceiverAddress:(NSString *)receiverAddress
                senderAddress:(NSString *)senderAddress
                      subject:(NSString *)subject
                         body:(NSString *)body
                 smtpHostName:(NSString *)hostName;

// Designated initializer
- (id)initWithReceiverAddresses:(NSArray *)receiverAddresses
                  senderAddress:(NSString *)senderAddress
                    ccAddresses:(NSArray *)ccAddresses
                   bccAddresses:(NSArray *)bccAddresses
                        subject:(NSString *)subject
                           body:(NSString *)body
                   smtpHostName:(NSString *)hostName
                       smtpPort:(unsigned int)port;

- (void)dealloc;

- (void)addReceiverAddress:(NSString *)receiverAddress;
- (NSMutableArray *)receiverAddresses;

- (void)setSenderAddress:(NSString *)senderAddress;
- (NSString *)senderAddress;

- (void)addCCAddress:(NSString *)ccAddress;
- (NSMutableArray *)ccAddresses;

- (void)addBCCAddress:(NSString *)bccAddress;
- (NSMutableArray *)bccAddresses;

- (void)setSubject:(NSString *)subject;
- (NSString *)subject;

- (void)setBody:(NSString *)body;
- (NSString *)body;

- (void)setSMTPHostName:(NSString *)hostName;
- (NSString *)smtpHostName;

- (void)setSMTPPort:(unsigned int)port;
- (unsigned int )smtpPort;

- (void)deliver;
    // Connects to the specified SMTP server and attempts to deliver the message.
    // Can raise.

@end


// Exception names

OmniMail_EXTERN NSString *OMSMTPException;
    // This exception will be returned if OMMailMessage encounters an unexpected reply from
    // the SMTP server.  For example, if the SMTP server cannot resolve one of the addresses
    // in your receiver list, it will return an error.  Unlike some mail clients, which allow
    // you to continue with your message, OMMailMessage immediately raises.
